@@ -17,4 +17,8 @@
 # NOTE: This file is deprecated and will be removed in a future version.
 # It only exists so that temporarely `from diffusers.pipelines import DiffusionPipeline` works
 
-from .pipelines import DiffusionPipeline, ImagePipelineOutput  # noqa: F401
+from .pipelines import (  # noqa: F401
+    DiffusionPipeline,
+    ImagePipelineOutput,
+    TextPipelineOutput,
+)
