@@ -1,5 +1,4 @@
 # Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
-# Copyright 2023 The HuggingFace Team. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-
 from typing import List, Optional, Tuple, Union
 
 import paddle
@@ -21,11 +19,11 @@ import paddle
 from ...utils import logging, randn_tensor
 from ..pipeline_utils import AudioPipelineOutput, DiffusionPipeline
 
-logger = logging.get_logger(__name__)  # pylint: disable=invalid-name
+logger = logging.get_logger(__name__)
 
 
 class DanceDiffusionPipeline(DiffusionPipeline):
-    r"""
+    """
     This model inherits from [`DiffusionPipeline`]. Check the superclass documentation for the generic methods the
     library implements for all the pipelines (such as downloading or saving, running on a particular device, etc.)
 
@@ -49,7 +47,7 @@ class DanceDiffusionPipeline(DiffusionPipeline):
         audio_length_in_s: Optional[float] = None,
         return_dict: bool = True,
     ) -> Union[AudioPipelineOutput, Tuple]:
-        r"""
+        """
         Args:
             batch_size (`int`, *optional*, defaults to 1):
                 The number of audio samples to generate.
@@ -60,7 +58,7 @@ class DanceDiffusionPipeline(DiffusionPipeline):
                 One or a list of paddle generator(s) to make generation deterministic.
             audio_length_in_s (`float`, *optional*, defaults to `self.unet.config.sample_size/self.unet.config.sample_rate`):
                 The length of the generated audio sample in seconds. Note that the output of the pipeline, *i.e.*
-                `sample_size`, will be `audio_length_in_s` * `self.unet.sample_rate`.
+                `sample_size`, will be `audio_length_in_s` * `self.unet.config.sample_rate`.
             return_dict (`bool`, *optional*, defaults to `True`):
                 Whether or not to return a [`~pipelines.AudioPipelineOutput`] instead of a plain tuple.
 
@@ -68,39 +66,30 @@ class DanceDiffusionPipeline(DiffusionPipeline):
             [`~pipelines.AudioPipelineOutput`] or `tuple`: [`~pipelines.utils.AudioPipelineOutput`] if `return_dict` is
             True, otherwise a `tuple. When returning a tuple, the first element is a list with the generated images.
         """
-
         if audio_length_in_s is None:
             audio_length_in_s = self.unet.config.sample_size / self.unet.config.sample_rate
-
-        sample_size = audio_length_in_s * self.unet.sample_rate
-
+        sample_size = audio_length_in_s * self.unet.config.sample_rate
         down_scale_factor = 2 ** len(self.unet.up_blocks)
         if sample_size < 3 * down_scale_factor:
             raise ValueError(
-                f"{audio_length_in_s} is too small. Make sure it's bigger or equal to"
-                f" {3 * down_scale_factor / self.unet.sample_rate}."
+                f"{audio_length_in_s} is too small. Make sure it's bigger or equal to {3 * down_scale_factor / self.unet.config.sample_rate}."
             )
-
         original_sample_size = int(sample_size)
         if sample_size % down_scale_factor != 0:
-            sample_size = ((audio_length_in_s * self.unet.sample_rate) // down_scale_factor + 1) * down_scale_factor
+            sample_size = (
+                audio_length_in_s * self.unet.config.sample_rate // down_scale_factor + 1
+            ) * down_scale_factor
             logger.info(
-                f"{audio_length_in_s} is increased to {sample_size / self.unet.sample_rate} so that it can be handled"
-                f" by the model. It will be cut to {original_sample_size / self.unet.sample_rate} after the denoising"
-                " process."
+                f"{audio_length_in_s} is increased to {sample_size / self.unet.config.sample_rate} so that it can be handled by the model. It will be cut to {original_sample_size / self.unet.config.sample_rate} after the denoising process."
             )
         sample_size = int(sample_size)
-
         dtype = self.unet.dtype
-        shape = [batch_size, self.unet.in_channels, sample_size]
+        shape = batch_size, self.unet.config.in_channels, sample_size
         if isinstance(generator, list) and len(generator) != batch_size:
             raise ValueError(
-                f"You have passed a list of generators of length {len(generator)}, but requested an effective batch"
-                f" size of {batch_size}. Make sure the batch size matches the length of the generators."
+                f"You have passed a list of generators of length {len(generator)}, but requested an effective batch size of {batch_size}. Make sure the batch size matches the length of the generators."
             )
-
         audio = randn_tensor(shape, generator=generator, dtype=dtype)
-
         # set step values
         self.scheduler.set_timesteps(num_inference_steps)
         # TODO donot cast dtype here
@@ -113,11 +102,8 @@ class DanceDiffusionPipeline(DiffusionPipeline):
             # 2. compute previous image: x_t -> t_t-1
             audio = self.scheduler.step(model_output, t, audio).prev_sample
 
-        audio = audio.clip(-1, 1).cast("float32").cpu().numpy()
-
+        audio = audio.clip(min=-1, max=1).astype(dtype="float32").cpu().numpy()
         audio = audio[:, :, :original_sample_size]
-
         if not return_dict:
             return (audio,)
-
         return AudioPipelineOutput(audios=audio)
