@@ -20,6 +20,7 @@ import unittest
 import numpy as np
 import paddle
 
+import ppdiffusers  # noqa F401
 from paddlenlp.transformers import XLMRobertaTokenizer
 from ppdiffusers import (
     AltDiffusionImg2ImgPipeline,
@@ -27,6 +28,7 @@ from ppdiffusers import (
     PNDMScheduler,
     UNet2DConditionModel,
 )
+from ppdiffusers.image_processor import VaeImageProcessor
 from ppdiffusers.pipelines.alt_diffusion.modeling_roberta_series import (
     RobertaSeriesConfig,
     RobertaSeriesModelWithTransformation,
@@ -124,6 +126,7 @@ class AltDiffusionImg2ImgPipelineFastTests(unittest.TestCase):
             safety_checker=None,
             feature_extractor=self.dummy_extractor,
         )
+        alt_pipe.image_processor = VaeImageProcessor(vae_scale_factor=alt_pipe.vae_scale_factor)
         alt_pipe.set_progress_bar_config(disable=None)
         prompt = "A painting of a squirrel eating a burger"
         generator = paddle.Generator().manual_seed(0)
@@ -150,7 +153,7 @@ class AltDiffusionImg2ImgPipelineFastTests(unittest.TestCase):
         image_from_tuple_slice = image_from_tuple[0, -3:, -3:, -1]
         assert image.shape == (1, 32, 32, 3)
         expected_slice = np.array(
-            [0.920333, 0.53369606, 0.56038886, 0.47739977, 0.18425128, 0.47001246, 0.5406687, 0.4329021, 0.6154301]
+            [0.48931587, 0.40102208, 0.49653798, 0.4203022, 0.34621224, 0.50789315, 0.41116416, 0.4933398, 0.5465742]
         )
         assert np.abs(image_slice.flatten() - expected_slice).max() < 0.005
         assert np.abs(image_from_tuple_slice.flatten() - expected_slice).max() < 0.005
@@ -176,6 +179,7 @@ class AltDiffusionImg2ImgPipelineFastTests(unittest.TestCase):
             safety_checker=None,
             feature_extractor=self.dummy_extractor,
         )
+        alt_pipe.image_processor = VaeImageProcessor(vae_scale_factor=alt_pipe.vae_scale_factor)
         alt_pipe.set_progress_bar_config(disable=None)
         prompt = "A painting of a squirrel eating a burger"
         generator = paddle.Generator().manual_seed(0)
@@ -204,7 +208,7 @@ class AltDiffusionImg2ImgPipelineFastTests(unittest.TestCase):
         expected_slice = np.array(
             [0.3251649, 0.3340174, 0.3418343, 0.32628638, 0.33462793, 0.3300547, 0.31628466, 0.3470268, 0.34273332]
         )
-        assert np.abs(image_slice.flatten() - expected_slice).max() < 0.001
+        assert np.abs(image_slice.flatten() - expected_slice).max() < 0.005
 
 
 @slow
