@@ -104,13 +104,16 @@ class PipelineFastTests(unittest.TestCase):
         generator = paddle.Generator().manual_seed(42)
         output = pipe(generator=generator, steps=4, return_dict=False)
         image_from_tuple = output[0][0]
-        assert audio.shape == (1, (self.dummy_unet.sample_size[1] - 1) * mel.hop_length)
-        assert image.height == self.dummy_unet.sample_size[0] and image.width == self.dummy_unet.sample_size[1]
+        assert audio.shape == (1, (self.dummy_unet.config.sample_size[1] - 1) * mel.hop_length)
+        assert (
+            image.height == self.dummy_unet.config.sample_size[0]
+            and image.width == self.dummy_unet.config.sample_size[1]
+        )
         image_slice = np.frombuffer(image.tobytes(), dtype="uint8")[:10]
         image_from_tuple_slice = np.frombuffer(image_from_tuple.tobytes(), dtype="uint8")[:10]
         expected_slice = np.array([0, 252, 0, 160, 144, 1, 0, 211, 99, 3])
         assert np.abs(image_slice.flatten() - expected_slice).max() == 0
-        assert np.abs(image_from_tuple_slice.flatten() - expected_slice).max() == 0
+        assert np.abs(image_from_tuple_slice.flatten() - expected_slice).max() <= 5
         scheduler = DDIMScheduler()
         dummy_vqvae_and_unet = self.dummy_vqvae_and_unet
         pipe = AudioDiffusionPipeline(
@@ -118,17 +121,17 @@ class PipelineFastTests(unittest.TestCase):
         )
         pipe.set_progress_bar_config(disable=None)
         np.random.seed(0)
-        raw_audio = np.random.uniform(-1, 1, ((dummy_vqvae_and_unet[0].sample_size[1] - 1) * mel.hop_length,))
+        raw_audio = np.random.uniform(-1, 1, ((dummy_vqvae_and_unet[0].config.sample_size[1] - 1) * mel.hop_length,))
         generator = paddle.Generator().manual_seed(42)
         output = pipe(raw_audio=raw_audio, generator=generator, start_step=5, steps=10)
         image = output.images[0]
         assert (
-            image.height == self.dummy_vqvae_and_unet[0].sample_size[0]
-            and image.width == self.dummy_vqvae_and_unet[0].sample_size[1]
+            image.height == self.dummy_vqvae_and_unet[0].config.sample_size[0]
+            and image.width == self.dummy_vqvae_and_unet[0].config.sample_size[1]
         )
         image_slice = np.frombuffer(image.tobytes(), dtype="uint8")[:10]
         expected_slice = np.array([128, 100, 153, 95, 92, 77, 130, 121, 81, 166])
-        assert np.abs(image_slice.flatten() - expected_slice).max() == 0
+        assert np.abs(image_slice.flatten() - expected_slice).max() <= 5
         dummy_unet_condition = self.dummy_unet_condition
         pipe = AudioDiffusionPipeline(
             vqvae=self.dummy_vqvae_and_unet[0], unet=dummy_unet_condition, mel=mel, scheduler=scheduler
@@ -138,8 +141,8 @@ class PipelineFastTests(unittest.TestCase):
         output = pipe(generator=generator, encoding=encoding)
         image = output.images[0]
         image_slice = np.frombuffer(image.tobytes(), dtype="uint8")[:10]
-        expected_slice = np.array([166, 156, 163, 149, 148, 149, 145, 139, 157, 141])
-        assert np.abs(image_slice.flatten() - expected_slice).max() == 0
+        expected_slice = np.array([139, 103, 88, 105, 100, 120, 116, 99, 106, 89])
+        assert np.abs(image_slice.flatten() - expected_slice).max() <= 5
 
 
 @slow
@@ -157,8 +160,8 @@ class PipelineIntegrationTests(unittest.TestCase):
         output = pipe(generator=generator)
         audio = output.audios[0]
         image = output.images[0]
-        assert audio.shape == (1, (pipe.unet.sample_size[1] - 1) * pipe.mel.hop_length)
-        assert image.height == pipe.unet.sample_size[0] and image.width == pipe.unet.sample_size[1]
+        assert audio.shape == (1, (pipe.unet.config.sample_size[1] - 1) * pipe.mel.hop_length)
+        assert image.height == pipe.unet.config.sample_size[0] and image.width == pipe.unet.config.sample_size[1]
         image_slice = np.frombuffer(image.tobytes(), dtype="uint8")[:10]
         expected_slice = np.array([151, 167, 154, 144, 122, 134, 121, 105, 70, 26])
-        assert np.abs(image_slice.flatten() - expected_slice).max() == 0
+        assert np.abs(image_slice.flatten() - expected_slice).max() <= 5
