@@ -1,5 +1,5 @@
-# coding=utf-8
-# Copyright 2022 HuggingFace Inc.
+# Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
+# Copyright 2023 The HuggingFace Team. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,10 +19,10 @@ import numpy as np
 import paddle
 
 from ppdiffusers import ScoreSdeVePipeline, ScoreSdeVeScheduler, UNet2DModel
-from ppdiffusers.utils.testing_utils import slow
+from ppdiffusers.utils.testing_utils import require_paddle, slow
 
 
-class ScoreSdeVePipelineFastTests(unittest.TestCase):
+class ScoreSdeVeipelineFastTests(unittest.TestCase):
     @property
     def dummy_uncond_unet(self):
         paddle.seed(0)
@@ -40,47 +40,34 @@ class ScoreSdeVePipelineFastTests(unittest.TestCase):
     def test_inference(self):
         unet = self.dummy_uncond_unet
         scheduler = ScoreSdeVeScheduler()
-
         sde_ve = ScoreSdeVePipeline(unet=unet, scheduler=scheduler)
         sde_ve.set_progress_bar_config(disable=None)
-
         generator = paddle.Generator().manual_seed(0)
-
         image = sde_ve(num_inference_steps=2, output_type="numpy", generator=generator).images
-
         generator = paddle.Generator().manual_seed(0)
-
         image_from_tuple = sde_ve(num_inference_steps=2, output_type="numpy", generator=generator, return_dict=False)[
             0
         ]
-
         image_slice = image[0, -3:, -3:, -1]
         image_from_tuple_slice = image_from_tuple[0, -3:, -3:, -1]
-
         assert image.shape == (1, 32, 32, 3)
         expected_slice = np.array([0.0, 1.0, 1.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0])
-        assert np.abs(image_slice.flatten() - expected_slice).max() < 1e-2
-        assert np.abs(image_from_tuple_slice.flatten() - expected_slice).max() < 1e-2
+        assert np.abs(image_slice.flatten() - expected_slice).max() < 0.01
+        assert np.abs(image_from_tuple_slice.flatten() - expected_slice).max() < 0.01
 
 
 @slow
+@require_paddle
 class ScoreSdeVePipelineIntegrationTests(unittest.TestCase):
     def test_inference(self):
         model_id = "google/ncsnpp-church-256"
         model = UNet2DModel.from_pretrained(model_id)
-
         scheduler = ScoreSdeVeScheduler.from_pretrained(model_id)
-
         sde_ve = ScoreSdeVePipeline(unet=model, scheduler=scheduler)
         sde_ve.set_progress_bar_config(disable=None)
-
         generator = paddle.Generator().manual_seed(0)
-
         image = sde_ve(num_inference_steps=10, output_type="numpy", generator=generator).images
-
         image_slice = image[0, -3:, -3:, -1]
-
         assert image.shape == (1, 256, 256, 3)
-
         expected_slice = np.array([1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 1.0])
-        assert np.abs(image_slice.flatten() - expected_slice).max() < 1e-2
+        assert np.abs(image_slice.flatten() - expected_slice).max() < 0.01
